==> avfs_1.0.1.orig.tar.gz: avfs-1.0.1/zlib/infcodes.c <==
/* infcodes.c -- process literals and length/distance pairs
 * Copyright (C) 1995-2002 Mark Adler
 * For conditions of distribution and use, see copyright notice in zlib.h 
 */

#include "zutil.h"
#include "inftrees.h"
#include "infblock.h"
#include "infcodes.h"
#include "infutil.h"
#include "inffast.h"

#include <assert.h>

/* simplify the use of the inflate_huft type with some defines */
#define exop word.what.Exop
#define bits word.what.Bits

typedef enum {        /* waiting for "i:"=input, "o:"=output, "x:"=nothing */
      START,    /* x: set up for LEN */
      LEN,      /* i: get length/literal/eob next */
      LENEXT,   /* i: getting length extra (have base) */
      DIST,     /* i: get distance next */
      DISTEXT,  /* i: getting distance extra */
      COPY,     /* o: copying bytes in window, waiting for space */
      LIT,      /* o: got literal, waiting for output space */
      WASH,     /* o: got eob, possibly still output waiting */
      END,      /* x: got eob and all data flushed */
      BADCODE}  /* x: got error */
inflate_codes_mode;

/* inflate codes private state */
struct inflate_codes_state {

  /* mode */
  inflate_codes_mode mode;      /* current inflate_codes mode */

  /* mode dependent information */
  uInt len;
  union {
    struct {
      inflate_huft *tree;       /* pointer into tree */
      uInt need;                /* bits needed */
    } code;             /* if LEN or DIST, where in tree */
    uInt lit;           /* if LIT, literal */
    struct {
      uInt get;                 /* bits to get for extra */
      uInt dist;                /* distance back to copy from */
    } copy;             /* if EXT or COPY, where and how much */
  } sub;                /* submode */

  /* mode independent information */
  Byte lbits;           /* ltree bits decoded per branch */
  Byte dbits;           /* dtree bits decoder per branch */
  inflate_huft *ltree;          /* literal/length/eob tree */
  inflate_huft *dtree;          /* distance tree */

};


inflate_codes_statef *inflate_codes_new(bl, bd, tl, td, z)
uInt bl, bd;
inflate_huft *tl;
inflate_huft *td; /* need separate declaration for Borland C++ */
z_streamp z;
{
  inflate_codes_statef *c;

  if ((c = (inflate_codes_statef *)
       ZALLOC(z,1,sizeof(struct inflate_codes_state))) != Z_NULL)
  {
    c->mode = START;
    c->lbits = (Byte)bl;
    c->dbits = (Byte)bd;
    c->ltree = tl;
    c->dtree = td;
    Tracev((stderr, "inflate:       codes new\n"));
  }
  return c;
}


int inflate_codes(s, z, r)
inflate_blocks_statef *s;
z_streamp z;
int r;
{
  uInt j;               /* temporary storage */
  inflate_huft *t;      /* temporary pointer */
  uInt e;               /* extra bits or operation */
  uLong b;              /* bit buffer */
  uInt k;               /* bits in bit buffer */
  Bytef *p;             /* input data pointer */
  uInt n;               /* bytes available there */
  Bytef *q;             /* output window write pointer */
  uInt m;               /* bytes to end of window or read pointer */
  Bytef *f;             /* pointer to copy strings from */
  inflate_codes_statef *c = s->sub.decode.codes;  /* codes state */

  /* copy input/output information to locals (UPDATE macro restores) */
  LOAD

  /* process input and output based on current state */
  while (1) switch (c->mode)
  {             /* waiting for "i:"=input, "o:"=output, "x:"=nothing */
    case START:         /* x: set up for LEN */
#ifndef SLOW
      if (m >= 258 && n >= 10)
      {
        UPDATE
        r = inflate_fast(c->lbits, c->dbits, c->ltree, c->dtree, s, z);
        LOAD
        if (r != Z_OK)
        {
          c->mode = r == Z_STREAM_END ? WASH : BADCODE;
          break;
        }
      }
#endif /* !SLOW */
      c->sub.code.need = c->lbits;
      c->sub.code.tree = c->ltree;
      c->mode = LEN;
    case LEN:           /* i: get length/literal/eob next */
      j = c->sub.code.need;
      NEEDBITS(j)
      t = c->sub.code.tree + ((uInt)b & inflate_mask[j]);
      DUMPBITS(t->bits)
      e = (uInt)(t->exop);
      if (e == 0)               /* literal */
      {
        c->sub.lit = t->base;
        Tracevv((stderr, t->base >= 0x20 && t->base < 0x7f ?
                 "inflate:         literal '%c'\n" :
                 "inflate:         literal 0x%02x\n", t->base));
        c->mode = LIT;
        break;
      }
      if (e & 16)               /* length */
      {
        c->sub.copy.get = e & 15;
        c->len = t->base;
        c->mode = LENEXT;
        break;
      }
      if ((e & 64) == 0)        /* next table */
      {
        c->sub.code.need = e;
        c->sub.code.tree = t + t->base;
        break;
      }
      if (e & 32)               /* end of block */
      {
        Tracevv((stderr, "inflate:         end of block\n"));
        c->mode = WASH;
        break;
      }
      c->mode = BADCODE;        /* invalid code */
      z->msg = (char*)"invalid literal/length code";
      r = Z_DATA_ERROR;
      LEAVE
    case LENEXT:        /* i: getting length extra (have base) */
      j = c->sub.copy.get;
      NEEDBITS(j)
      c->len += (uInt)b & inflate_mask[j];
      DUMPBITS(j)
      c->sub.code.need = c->dbits;
      c->sub.code.tree = c->dtree;
      Tracevv((stderr, "inflate:         length %u\n", c->len));
      c->mode = DIST;
    case DIST:          /* i: get distance next */
      j = c->sub.code.need;
      NEEDBITS(j)
      t = c->sub.code.tree + ((uInt)b & inflate_mask[j]);
      DUMPBITS(t->bits)
      e = (uInt)(t->exop);
      if (e & 16)               /* distance */
      {
        c->sub.copy.get = e & 15;
        c->sub.copy.dist = t->base;
        c->mode = DISTEXT;
        break;
      }
      if ((e & 64) == 0)        /* next table */
      {
        c->sub.code.need = e;
        c->sub.code.tree = t + t->base;
        break;
      }
      c->mode = BADCODE;        /* invalid code */
      z->msg = (char*)"invalid distance code";
      r = Z_DATA_ERROR;
      LEAVE
    case DISTEXT:       /* i: getting distance extra */
      j = c->sub.copy.get;
      NEEDBITS(j)
      c->sub.copy.dist += (uInt)b & inflate_mask[j];
      DUMPBITS(j)
      Tracevv((stderr, "inflate:         distance %u\n", c->sub.copy.dist));
      c->mode = COPY;
    case COPY:          /* o: copying bytes in window, waiting for space */
      f = q - c->sub.copy.dist;
      while (f < s->window)             /* modulo window size-"while" instead */
        f += s->end - s->window;        /* of "if" handles invalid distances */
      while (c->len)
      {
        NEEDOUT
        OUTBYTE(*f++)
        if (f == s->end)
          f = s->window;
        c->len--;
      }
      c->mode = START;
      break;
    case LIT:           /* o: got literal, waiting for output space */
      NEEDOUT
      OUTBYTE(c->sub.lit)
      c->mode = START;
      break;
    case WASH:          /* o: got eob, possibly more output */
      if (k > 7)        /* return unused byte, if any */
      {
        Assert(k < 16, "inflate_codes grabbed too many bytes")
        k -= 8;
        n++;
        p--;            /* can always return one */
      }
      FLUSH
      if (s->read != s->write)
        LEAVE
      c->mode = END;
    case END:
      r = Z_STREAM_END;
      LEAVE
    case BADCODE:       /* x: got error */
      r = Z_DATA_ERROR;
      LEAVE
    default:
      r = Z_STREAM_ERROR;
      LEAVE
  }
#ifdef NEED_DUMMY_RETURN
  return Z_STREAM_ERROR;  /* Some dumb compilers complain without this */
#endif
}


void inflate_codes_free(c, z)
inflate_codes_statef *c;
z_streamp z;
{
  ZFREE(z, c);
  Tracev((stderr, "inflate:       codes free\n"));
}

int inflate_codes_save(bufp, at, s, z)
char **bufp;
int at;
inflate_blocks_statef *s;
z_streamp z;
{
  char *buf = *bufp;
  inflate_codes_statef *c = s->sub.decode.codes;
  struct inflate_codes_state ccpy = *c;
  char fixed_tree = 0;

  if(c->mode == LEN || c->mode == DIST) {
    assert(c->sub.code.tree >= s->hufts && c->sub.code.tree < s->hufts + MANY);
    ccpy.sub.code.tree = (inflate_huft *) ((long) c->sub.code.tree - (long) s->hufts);
  }

  if(c->ltree >= s->hufts && c->ltree < s->hufts + MANY) {
    ccpy.ltree = (inflate_huft *) ((long) c->ltree - (long) s->hufts);
    
    assert(c->dtree >= s->hufts && c->dtree < s->hufts + MANY);
    ccpy.dtree = (inflate_huft *) ((long) c->dtree - (long) s->hufts);
  } else {
    /* this is an assumption, it will be checked in restore */
    fixed_tree = 1;
  }
  
  *bufp = buf = realloc(buf, at + sizeof(struct inflate_codes_state) + 1);
  if(buf == NULL)
    return Z_MEM_ERROR;
  
  memcpy(buf + at, &ccpy, sizeof(struct inflate_codes_state));
  at += sizeof(struct inflate_codes_state);

  buf[at++] = fixed_tree;
  
  return at;
}

inflate_codes_statef *inflate_codes_restore(bufp, s, z)
  char **bufp;
  inflate_blocks_statef *s;
  z_streamp z;
{
  char *buf = *bufp;
  inflate_codes_statef *c;
  char fixed_tree = 0;

  if ((c = (inflate_codes_statef *)
       ZALLOC(z,1,sizeof(struct inflate_codes_state))) == Z_NULL)
    return c;
  
  memcpy(c, buf, sizeof(struct inflate_codes_state));

  fixed_tree = buf[sizeof(struct inflate_codes_state)];

  if(fixed_tree == 0) {
    c->ltree = (inflate_huft *) ((long) s->hufts + (long) c->ltree);
    c->dtree = (inflate_huft *) ((long) s->hufts + (long) c->dtree);
  } else if(fixed_tree == 1) {
    Byte old_lbits = c->lbits;
    Byte old_dbits = c->dbits;
    inflate_huft *old_ltree = c->ltree;
    inflate_huft *old_dtree = c->dtree;
    uInt bl, bd;

    if(inflate_trees_fixed(&bl, &bd, &c->ltree, &c->dtree, z) != Z_OK) {
      ZFREE(z, c);
      return Z_NULL;
    }

    c->lbits = (Byte)bl;
    c->dbits = (Byte)bd;

    /* it was an assumption that fixed trees are used
       if one of the following conditions is false we were wrong */
    if(c->lbits != old_lbits || c->dbits != old_dbits ||
       c->ltree != old_ltree || c->dtree != old_dtree) {
      ZFREE(z, c);
      return Z_NULL;
    }
  } else {
    return Z_NULL;
  }

  buf += sizeof(struct inflate_codes_state) + 1;
  
  *bufp = buf;

  if(c->mode == LEN || c->mode == DIST)
    c->sub.code.tree = (inflate_huft *) ((long) s->hufts + (long) c->sub.code.tree);

  return c;
}

==> avfs_1.1.4.orig.tar.gz: avfs-1.1.4/zlib/infcodes.c <==
/* infcodes.c -- process literals and length/distance pairs
 * Copyright (C) 1995-2002 Mark Adler
 * For conditions of distribution and use, see copyright notice in zlib.h 
 */

#include "zutil.h"
#include "inftrees.h"
#include "infblock.h"
#include "infcodes.h"
#include "infutil.h"
#include "inffast.h"

#include <assert.h>

/* simplify the use of the inflate_huft type with some defines */
#define exop word.what.Exop
#define bits word.what.Bits

typedef enum {        /* waiting for "i:"=input, "o:"=output, "x:"=nothing */
      START,    /* x: set up for LEN */
      LEN,      /* i: get length/literal/eob next */
      LENEXT,   /* i: getting length extra (have base) */
      DIST,     /* i: get distance next */
      DISTEXT,  /* i: getting distance extra */
      COPY,     /* o: copying bytes in window, waiting for space */
      LIT,      /* o: got literal, waiting for output space */
      WASH,     /* o: got eob, possibly still output waiting */
      END,      /* x: got eob and all data flushed */
      BADCODE}  /* x: got error */
inflate_codes_mode;

/* inflate codes private state */
struct inflate_codes_state {

  /* mode */
  inflate_codes_mode mode;      /* current inflate_codes mode */

  /* mode dependent information */
  uInt len;
  union {
    struct {
      inflate_huft *tree;       /* pointer into tree */
      uInt need;                /* bits needed */
    } code;             /* if LEN or DIST, where in tree */
    uInt lit;           /* if LIT, literal */
    struct {
      uInt get;                 /* bits to get for extra */
      uInt dist;                /* distance back to copy from */
    } copy;             /* if EXT or COPY, where and how much */
  } sub;                /* submode */

  /* mode independent information */
  Byte lbits;           /* ltree bits decoded per branch */
  Byte dbits;           /* dtree bits decoder per branch */
  inflate_huft *ltree;          /* literal/length/eob tree */
  inflate_huft *dtree;          /* distance tree */

};


inflate_codes_statef *inflate_codes_new(bl, bd, tl, td, z)
uInt bl, bd;
inflate_huft *tl;
inflate_huft *td; /* need separate declaration for Borland C++ */
z_streamp z;
{
  inflate_codes_statef *c;

  if ((c = (inflate_codes_statef *)
       ZALLOC(z,1,sizeof(struct inflate_codes_state))) != Z_NULL)
  {
    c->mode = START;
    c->lbits = (Byte)bl;
    c->dbits = (Byte)bd;
    c->ltree = tl;
    c->dtree = td;
    Tracev((stderr, "inflate:       codes new\n"));
  }
  return c;
}


int inflate_codes(s, z, r)
inflate_blocks_statef *s;
z_streamp z;
int r;
{
  uInt j;               /* temporary storage */
  inflate_huft *t;      /* temporary pointer */
  uInt e;               /* extra bits or operation */
  uLong b;              /* bit buffer */
  uInt k;               /* bits in bit buffer */
  Bytef *p;             /* input data pointer */
  uInt n;               /* bytes available there */
  Bytef *q;             /* output window write pointer */
  uInt m;               /* bytes to end of window or read pointer */
  Bytef *f;             /* pointer to copy strings from */
  inflate_codes_statef *c = s->sub.decode.codes;  /* codes state */

  /* copy input/output information to locals (UPDATE macro restores) */
  LOAD

  /* process input and output based on current state */
  while (1) switch (c->mode)
  {             /* waiting for "i:"=input, "o:"=output, "x:"=nothing */
    case START:         /* x: set up for LEN */
#ifndef SLOW
      if (m >= 258 && n >= 10)
      {
        UPDATE
        r = inflate_fast(c->lbits, c->dbits, c->ltree, c->dtree, s, z);
        LOAD
        if (r != Z_OK)
        {
          c->mode = r == Z_STREAM_END ? WASH : BADCODE;
          break;
        }
      }
#endif /* !SLOW */
      c->sub.code.need = c->lbits;
      c->sub.code.tree = c->ltree;
      c->mode = LEN;
    case LEN:           /* i: get length/literal/eob next */
      j = c->sub.code.need;
      NEEDBITS(j)
      t = c->sub.code.tree + ((uInt)b & inflate_mask[j]);
      DUMPBITS(t->bits)
      e = (uInt)(t->exop);
      if (e == 0)               /* literal */
      {
        c->sub.lit = t->base;
        Tracevv((stderr, t->base >= 0x20 && t->base < 0x7f ?
                 "inflate:         literal '%c'\n" :
                 "inflate:         literal 0x%02x\n", t->base));
        c->mode = LIT;
        break;
      }
      if (e & 16)               /* length */
      {
        c->sub.copy.get = e & 15;
        c->len = t->base;
        c->mode = LENEXT;
        break;
      }
      if ((e & 64) == 0)        /* next table */
      {
        c->sub.code.need = e;
        c->sub.code.tree = t + t->base;
        break;
      }
      if (e & 32)               /* end of block */
      {
        Tracevv((stderr, "inflate:         end of block\n"));
        c->mode = WASH;
        break;
      }
      c->mode = BADCODE;        /* invalid code */
      z->msg = (char*)"invalid literal/length code";
      r = Z_DATA_ERROR;
      LEAVE
    case LENEXT:        /* i: getting length extra (have base) */
      j = c->sub.copy.get;
      NEEDBITS(j)
      c->len += (uInt)b & inflate_mask[j];
      DUMPBITS(j)
      c->sub.code.need = c->dbits;
      c->sub.code.tree = c->dtree;
      Tracevv((stderr, "inflate:         length %u\n", c->len));
      c->mode = DIST;
    case DIST:          /* i: get distance next */
      j = c->sub.code.need;
      NEEDBITS(j)
      t = c->sub.code.tree + ((uInt)b & inflate_mask[j]);
      DUMPBITS(t->bits)
      e = (uInt)(t->exop);
      if (e & 16)               /* distance */
      {
        c->sub.copy.get = e & 15;
        c->sub.copy.dist = t->base;
        c->mode = DISTEXT;
        break;
      }
      if ((e & 64) == 0)        /* next table */
      {
        c->sub.code.need = e;
        c->sub.code.tree = t + t->base;
        break;
      }
      c->mode = BADCODE;        /* invalid code */
      z->msg = (char*)"invalid distance code";
      r = Z_DATA_ERROR;
      LEAVE
    case DISTEXT:       /* i: getting distance extra */
      j = c->sub.copy.get;
      NEEDBITS(j)
      c->sub.copy.dist += (uInt)b & inflate_mask[j];
      DUMPBITS(j)
      Tracevv((stderr, "inflate:         distance %u\n", c->sub.copy.dist));
      c->mode = COPY;
    case COPY:          /* o: copying bytes in window, waiting for space */
      f = q - c->sub.copy.dist;
      while (f < s->window)             /* modulo window size-"while" instead */
        f += s->end - s->window;        /* of "if" handles invalid distances */
      while (c->len)
      {
        NEEDOUT
        OUTBYTE(*f++)
        if (f == s->end)
          f = s->window;
        c->len--;
      }
      c->mode = START;
      break;
    case LIT:           /* o: got literal, waiting for output space */
      NEEDOUT
      OUTBYTE(c->sub.lit)
      c->mode = START;
      break;
    case WASH:          /* o: got eob, possibly more output */
      if (k > 7)        /* return unused byte, if any */
      {
        Assert(k < 16, "inflate_codes grabbed too many bytes")
        k -= 8;
        n++;
        p--;            /* can always return one */
      }
      FLUSH
      if (s->read != s->write)
        LEAVE
      c->mode = END;
    case END:
      r = Z_STREAM_END;
      LEAVE
    case BADCODE:       /* x: got error */
      r = Z_DATA_ERROR;
      LEAVE
    default:
      r = Z_STREAM_ERROR;
      LEAVE
  }
#ifdef NEED_DUMMY_RETURN
  return Z_STREAM_ERROR;  /* Some dumb compilers complain without this */
#endif
}


void inflate_codes_free(c, z)
inflate_codes_statef *c;
z_streamp z;
{
  ZFREE(z, c);
  Tracev((stderr, "inflate:       codes free\n"));
}

int inflate_codes_save(bufp, at, s, z)
char **bufp;
int at;
inflate_blocks_statef *s;
z_streamp z;
{
  char *buf = *bufp;
  inflate_codes_statef *c = s->sub.decode.codes;
  struct inflate_codes_state ccpy = *c;
  char fixed_tree = 0;  /* this is a flag for the restore code to
                           indicate that a tree pointer points to a
                           fixed table instead of dynamic one.  Bit 0
                           is for the c->ltree/dtree, bit 1 for
                           c->sub.code.tree
                        */

  if(c->mode == LEN || c->mode == DIST) {
      if(c->sub.code.tree >= s->hufts && c->sub.code.tree < s->hufts + MANY) {
          ccpy.sub.code.tree = (inflate_huft *) ((long) c->sub.code.tree - (long) s->hufts);
      } else {
          /* this is an assumption, it will be checked in restore */
          fixed_tree |= 2;
      }
  }

  if(c->ltree >= s->hufts && c->ltree < s->hufts + MANY) {
    ccpy.ltree = (inflate_huft *) ((long) c->ltree - (long) s->hufts);
    
    assert(c->dtree >= s->hufts && c->dtree < s->hufts + MANY);
    ccpy.dtree = (inflate_huft *) ((long) c->dtree - (long) s->hufts);
  } else {
    /* this is an assumption, it will be checked in restore */
    fixed_tree |= 1;
  }
  
  *bufp = buf = realloc(buf, at + sizeof(struct inflate_codes_state) + 1);
  if(buf == NULL)
    return Z_MEM_ERROR;
  
  memcpy(buf + at, &ccpy, sizeof(struct inflate_codes_state));
  at += sizeof(struct inflate_codes_state);

  buf[at++] = fixed_tree;
  
  return at;
}

inflate_codes_statef *inflate_codes_restore(bufp, s, z)
  char **bufp;
  inflate_blocks_statef *s;
  z_streamp z;
{
  char *buf = *bufp;
  inflate_codes_statef *c;
  char fixed_tree = 0;

  if ((c = (inflate_codes_statef *)
       ZALLOC(z,1,sizeof(struct inflate_codes_state))) == Z_NULL)
    return c;
  
  memcpy(c, buf, sizeof(struct inflate_codes_state));

  fixed_tree = buf[sizeof(struct inflate_codes_state)];

  if( (fixed_tree & 1) == 0) {
    c->ltree = (inflate_huft *) ((long) s->hufts + (long) c->ltree);
    c->dtree = (inflate_huft *) ((long) s->hufts + (long) c->dtree);
  } else if( (fixed_tree & 1) == 1) {
    Byte old_lbits = c->lbits;
    Byte old_dbits = c->dbits;
    inflate_huft *old_ltree = c->ltree;
    inflate_huft *old_dtree = c->dtree;
    uInt bl, bd;

    if(inflate_trees_fixed(&bl, &bd, &c->ltree, &c->dtree, z) != Z_OK) {
      ZFREE(z, c);
      return Z_NULL;
    }

    c->lbits = (Byte)bl;
    c->dbits = (Byte)bd;

    /* it was an assumption that fixed trees are used
       if one of the following conditions is false we were wrong */
    if(c->lbits != old_lbits || c->dbits != old_dbits ||
       c->ltree != old_ltree || c->dtree != old_dtree) {
      ZFREE(z, c);
      return Z_NULL;
    }
  } else {
    return Z_NULL;
  }

  buf += sizeof(struct inflate_codes_state) + 1;
  
  *bufp = buf;

  if(c->mode == LEN || c->mode == DIST) {
      if (fixed_tree & 2) {
          /* c->sub.code.tree is probably also a fixed tree */
          Byte old_lbits = c->lbits;
          inflate_huft *old_ltree = c->ltree;
          inflate_huft *temp_dtree;
          uInt bl, bd;

          if(inflate_trees_fixed(&bl, &bd, &c->sub.code.tree, &temp_dtree, z) != Z_OK) {
              ZFREE(z, c);
              return Z_NULL;
          }

          c->sub.code.need = (Byte)bl;

          /* it was an assumption that fixed trees are used
             if one of the following conditions is false we were wrong */
          if (c->sub.code.tree != old_ltree ||
              c->sub.code.need != old_lbits) {
              ZFREE(z, c);
              return Z_NULL;
          }
      } else {
          c->sub.code.tree = (inflate_huft *) ((long) s->hufts + (long) c->sub.code.tree);
      }
  }

  return c;
}
